
var DrumProg = function(serial)
{
	this._serial = serial;
	this._port = null;

	this._RESET_PIN = 'dtr';
	this._BOOT0_PIN = 'rts';

	this._test = 0;

	this._sampleCount = 0;
	this._sampleIdxProgramming = 0;
}

/**
 * Finds compatible programmer
 * @param callback Returns error if present
 * @return void
 */
DrumProg.prototype.Find = function(callback) 
{
	var self = this;

	self._serial.ListPorts(function(err, ports) {
		var i;

		if(err) {
			if(callback && typeof callback === 'function')
				callback(err);
			return;
		}

		// Find Erica programmer
		for(i=0; i<ports.length; i++) {
			if((typeof ports[i].displayName != 'undefined' &&
				ports[i].displayName.indexOf('Erica') > -1 &&
				ports[i].path.indexOf('tty') > -1) || (ports[i].vendorId == 1027 && ports[i].productId == 24597)) {
				
				self._port = ports[i];
				callback(null);
				return;
			}
		}

		// If nothing found
		callback('Erica Synth Programmer not found!');
	});
}




DrumProg.prototype.Connect = function(callback) 
{
	var self = this;

	if(!self._port) {
		if(callback && typeof callback === 'function') {
			callback('No port!');
		}
		return;
	}

	// If not connected, connect
	if(!self._serial.IsOpen()) {
		self._serial.Open(self._port.path, function(err) {
			if(err) {
				if(callback && typeof callback === 'function') {
					callback(err);
				}
				return;
			}

			setTimeout(function() {
				self.Connect(callback) } , 100);
		});
	} 
	// Send programming start command
	else {
		self._serial.Write('erica', function(err) {
			if(!err) {
				self._serial.Read(1000, function(err, result) {
					if(err)
						return (callback && typeof callback === 'function' && callback('Connection error: ' + err));
// 
					if(callback && typeof callback === 'function') {
						console.log('Connected! ' + String.fromCharCode.apply(null, new Uint8Array(result)));
						callback(null);
					}
				});
			} else if(callback && typeof callback === 'function'){
				callback('Connection error: ' + err);
			}
		});
	}
}


DrumProg.prototype.Disconnect = function(callback) 
{
	var self = this;

	self._serial.Close(callback);
}


DrumProg.prototype.resetDevice = function(callback)
{
	var self = this;
	var signal = {};

	if(!self._serial.IsOpen())
		return (typeof callback === 'function' && callback('Port must be open!'));


	signal[self.RESET_PIN] = true;
	self._serial.control({ dtr : false }, function(err) {
		signal[self.RESET_PIN] = false;

		setTimeout(function(){
			self._serial.control({ dtr : true }, function(err) {

				if(callback && typeof callback === 'function') {
					// Wait until boots???
					setTimeout(callback, 100);
				}
			});
		}, 10);

	});
}


DrumProg.prototype.TestWrite = function() 
{
	var self = this;

	self._serial.Write('erica', function(err) {
			if(!err) {
				console.log('Sent: ' + self._test);
				self._test++;

				if(self._test < 31000) {
					setTimeout(function() {
						self.TestWrite();
					},5);
				}
			}
	});
}



DrumProg.prototype.WaitDevice = function() 
{

}

DrumProg.prototype.SendCmdData = function(cmd, address, size, callback) 
{
	var self = this;

	if(!self._serial.IsOpen())
		return (typeof callback === 'function' && callback('Port must be open!'));


	var output = new ArrayBuffer(9);	// CMD(1) + ADDR(4) + SIZE(4)
	var outputView = new DataView(output);

	outputView.setUint8(0, cmd);
	outputView.setUint32(1, address, true);
	outputView.setUint32(5, size, true);

	// DEBUG
	//console.log(new Uint8Array(output));

	self._serial.Write(output, function(err) {
		if(err)
			return (typeof callback === 'function' && callback(err));

		typeof callback === 'function' && callback(null);
	});
}


DrumProg.prototype.ReadResponse = function(timeout, callback) 
{
	var self = this;

	if(!self._serial.IsOpen())
		return (typeof callback === 'function' && callback('Port must be open!'));

	self._serial.Read(timeout, function(err, result) {
		if(err)
			return (callback && typeof callback === 'function' && callback(err));

		if(callback && typeof callback === 'function') {
			var resp = String.fromCharCode.apply(null, new Uint8Array(result));

			// console.log('Response: ', resp);
			if(resp.indexOf('ok') > -1)
				callback(null);
			else
				typeof callback === 'function' && callback('Response: ' + String.fromCharCode.apply(null, new Uint8Array(result)));
		}
	});
}

DrumProg.prototype.EraseFlash = function(callback)
{
	var self = this;

	if(!self._serial.IsOpen())
		return (typeof callback === 'function' && callback('Port must be open!'));

	// Send Erase comand
	self.SendCmdData(('e').charCodeAt(0), 0, 0, function(err) {
		if(err)
			return (typeof callback === 'function' && callback(err));

		// Wait for response
		self.ReadResponse(120000, function(err) {
			if(err)
				return (typeof callback === 'function' && callback(err));

			//console.log('Erase cycle complete!');
			typeof callback === 'function' && callback(null);
		});
	});
	// typeof callback === 'function' && callback(null);
}

DrumProg.prototype.ExitProgramming = function(callback) 
{

}


DrumProg.prototype.ProgramVerifyPage = function(cmdByte, address, offset, sample, statusCallback, callback)
{
	var self = this;

	// Send command
	self.SendCmdData(cmdByte, address, 256, function(err) {
		if(err)
			return (typeof callback === 'function' && callback('CMD error: ' + err + ' for ' + sample.name + ' at addr: ' + address + ' offset: ' + offset));
	
		// Send sample data
		self._serial.Write(sample.buffer.slice(offset, offset+256), function(err) {
			if(err)
				return (typeof callback === 'function' && callback('WRITE error: ' + err + ' for ' + sample.name + ' at addr: ' + address + ' offset: ' + offset));

			// Wait for response
			self.ReadResponse(100, function(err) {
				if(err) {

					self.ProgramVerifyPage(cmdByte, address, offset, sample, statusCallback, callback);
					return;
					//return (typeof callback === 'function' && callback('RESP error: ' + err + ' for ' + sample.name + ' at addr: ' + address + ' offset: ' + offset));
				}

					
				// Increase offset and address
				address += 256;
				offset += 256;

				if(offset == sample.buffer.byteLength) {
					typeof callback === 'function' && callback(null);
				} else {
					typeof statusCallback === 'function' && statusCallback('program', sample.name, offset/sample.buffer.byteLength*100);

					// Program Next Page
					self.ProgramVerifyPage(cmdByte, address, offset, sample, statusCallback, callback);
				}

			});
		});
	});
}


DrumProg.prototype.ProgramVerifySample = function(cmdByte, address, sampleOffset, samples, statusCallback, callback) 
{
	var self = this;
	
	self.ProgramVerifyPage(cmdByte, address, 0, samples[sampleOffset], statusCallback, function(err) {

		if(err)
			return (typeof callback === 'function' && callback(err));


		// Increase sample offset and address
		address += samples[sampleOffset].buffer.byteLength;
		sampleOffset++;

		self._sampleIdxProgramming = sampleOffset;

		if(sampleOffset == samples.length) {
			typeof callback === 'function' && callback(null);
		} else {
			// Program Next sample
			self.ProgramVerifySample(cmdByte, address, sampleOffset, samples, statusCallback, callback);
		}
	});
}


DrumProg.prototype.ProgramVerifyInfoData = function(cmdByte, samples, statusCallback, callback)
{
	var self = this;
	var infoData = new ArrayBuffer(4096);
	var infoDataView = new DataView(infoData);

	// Compose info data
	var offset = 0;
	var address = 4096;
	for(var i=0; i<samples.length; i++) {
		infoDataView.setUint32(offset, address, true);
		offset += 4;
		infoDataView.setUint32(offset, samples[i].buffer.byteLength, true);
		offset += 4;
		address += samples[i].buffer.byteLength;
	}

	// DEBUG
	// console.log(new Uint32Array(infoData));
	// console.log(new Uint8Array(infoData));


	// Compose sample like packet
	var infoDataSample = {
		name : 'Info Data',
		buffer : infoData
	};

	
	// Program/Verify Info Data
	self.ProgramVerifyPage(cmdByte, 0, 0, infoDataSample, statusCallback, function(err) {

		if(err)
			return (typeof callback === 'function' && callback(err));

		// Finished
		typeof callback === 'function' && callback(null);
	});
}


DrumProg.prototype.ProgramVerifyAllSamples = function(cmd, samples, statusCallback, callback)
{
	var self = this;
	var cmdByte;

	// Set if data will be programmed or verified
	if(cmd == 'program')
		cmdByte = ('p').charCodeAt(0);
	else
		cmdByte = ('v').charCodeAt(0);


	self._sampleCount = samples.length;
	this._sampleIdxProgramming = 0;

	// Progra/Verify Info data
	self.ProgramVerifyInfoData(cmdByte, samples, statusCallback, function(err) {
		if(err) 
			return (typeof callback === 'function' && callback(err));


		// Program/Verify all samples
		self.ProgramVerifySample(cmdByte, 4096, 0, samples, statusCallback, function(err) {
			if(err) 
				return (typeof callback === 'function' && callback(err));

			callback(null);
		});
	});
}


DrumProg.prototype.ProgramStatus = function() {
	var self = this;
	
	return (self._sampleIdxProgramming/self._sampleCount*100).toFixed();
}

