var Serial = function()
{
	this.handle = null;

	this.readCallback = null;
	this.errorCallback = null;

	this.inputBuffer = new Uint8Array(512);
	this.inputBufferIdx = 0;
	this.readBufferTimerId = null;
	this.readTimeoutTimerId = null;
}


Serial.prototype.Init = function() {
	var self = this;

	chrome.serial.onReceive.addListener(self.onDataReceived.bind(self));
	chrome.serial.onReceiveError.addListener(self.onDataReceivedError.bind(self));
}


Serial.prototype.IsOpen = function() {
	return (this.handle !== null);
}


Serial.prototype.ListPorts = function(callback) {
	chrome.serial.getDevices(function(ports) {
		
		if(!ports || ports.length === 0) {
			return (typeof callback === 'function' && callback('No ports found!'));
		}

		typeof callback === 'function' && callback(null, ports);
	});
}


Serial.prototype.Open = function(path, callback) 
{
	var self = this;

	// Check if port already open
	chrome.serial.getConnections(function(connectionInfos) {
		
		if(connectionInfos.length > 0) {
			console.log('Reused connection!');
			self.handle = connectionInfos[0].connectionId;

			typeof callback === 'function' && callback(null);
		} else {
			chrome.serial.connect(path, {
				bitrate: 921600,
			}, 
			function(connectionInfo) {
				if(!connectionInfo) {
					return (typeof callback === 'function' && callback("Could not open port " + path));
				}

				self.handle = connectionInfo.connectionId;

				console.log('Connection speed: ', connectionInfo.bitrate);

				if(self.handle !== -1) {
					typeof callback === 'function' && callback(null);
					
				} else {
					typeof callback === 'function' && callback("Failed to open port " + path)
				}
			});
		}
	});


	// chrome.serial.connect(path, {
	// 	bitrate: 921600,
	// }, 
	// function(connectionInfo) {
	// 	if(!connectionInfo) {
	// 		return (typeof callback === 'function' && callback("Could not open port " + path));
	// 	}

	// 	self.handle = connectionInfo.connectionId;

	// 	console.log('Connection speed: ', connectionInfo.bitrate);

	// 	if(self.handle !== -1) {
	// 		typeof callback === 'function' && callback(null);
			
	// 	} else {
	// 		typeof callback === 'function' && callback("Failed to open port " + path)
	// 	}
	// });
}


Serial.prototype.Close = function(callback) {
	var self = this;

	if(self.handle) {
		chrome.serial.disconnect(self.handle, function(result) {
			if(result) {
				self.handle = null;
				typeof callback === 'function' && callback(null);
			} else {
				typeof callback === 'function' && callback("Could not close port!");
			}
		});
	}
}


Serial.prototype.Write = function(data, callback) {
	var self = this;
	var dataToSend = null;

	if(!self.handle) {
		return (typeof callback === 'function' && callback('Port is closed!'));
	}

	if(typeof data === 'string') {
		dataToSend = str2ab(data);
	} else {
		dataToSend = data;
	}

	// console.log('Data to Send: ', dataToSend);

	chrome.serial.send(self.handle, dataToSend, function(sendInfo) {
		if(sendInfo.error) {
			console.log(sendInfo.error);

			typeof callback === 'function' && callback(sendInfo.error);
		} else if(sendInfo.bytesSent > 0) {
			
			typeof callback === 'function' && callback(null);
		}
	});
	
}


Serial.prototype.Read = function(timeout, callback) {
	var self = this;

	if(!self.handle) {
		return (callback && typeof callback === 'function' && callback('Port is closed!'));
	}

	if(callback && typeof callback === 'function') {
		self.readCallback = callback;

		self.readTimeoutTimerId = setTimeout(function() {

			// var resp = String.fromCharCode.apply(null, self.inputBuffer.subarray(0, self.inputBufferIdx));
			// console.log('Response: ', resp);

			callback('Read timeout!');
		}, timeout);
	}
}


Serial.prototype.control = function(signals, callback) {
	if(!this.handle)
		return (callback && typeof callback === 'function' && callback('Port is closed!'));

	chrome.serial.setControlSignals(this.handle, signals, function(result) {
		if(result)
			return (callback && typeof callback === 'function' && callback(null));
		else
			return (callback && typeof callback === 'function' && callback('Could not set control signals!'));
	});
}


Serial.prototype.onDataReceived = function(info) {
	var self = this;

	if(info.connectionId === self.handle && info.data) {

		if(self.readBufferTimerId) {
			clearTimeout(this.readBufferTimerId);
		}

		self.inputBuffer.set(new Uint8Array(info.data), self.inputBufferIdx);
		self.inputBufferIdx += info.data.byteLength;

		if(self.readCallback) {
			// self.readBufferTimerId = setTimeout(function() {
			// 	clearTimeout(self.readTimeoutTimerId);

			// 	self.readCallback(null, self.inputBuffer.subarray(0, self.inputBufferIdx));
			// 	self.inputBufferIdx = 0;
			// }, 2);

			if(self.inputBuffer.subarray(0, self.inputBufferIdx).includes(('\n').charCodeAt(0))) {
				// console.log(self.inputBuffer.subarray(0, self.inputBufferIdx));

				clearTimeout(self.readTimeoutTimerId);
				self.readCallback(null, self.inputBuffer.subarray(0, self.inputBufferIdx));
				self.inputBufferIdx = 0;
			}
		}
	}
}


Serial.prototype.onDataReceivedError = function(info) {
	var self = this;

	console.log('Serial Error: ', info);
}
