app.factory('programService', [function() {

	var self = {};
	
	self._serial = null;
	self._drum = null;

	self._connected = false;
	
	self.Init = function() {
		// Initialize serial port
		self._serial = new Serial();
		self._serial.Init();

		// Initialize Drum Module programming
		self._drum = new DrumProg(self._serial);
	}


	self.FindProgrammer = function(callback) {

		self._drum.Find(function(err) {
			if(err) {
				setTimeout(function() {
					self.FindProgrammer(callback);
				}, 5000);
				return (typeof callback === 'function' && callback(err));
			}

			typeof callback === 'function' && callback(null);
		});
	}


	self.ConnectToDevice = function(callback) {
		
		self._drum.Connect(function(err) {
			if(err) {
				console.log(err, ' Reseting device & trying to connect...');

				// Reset and try again to Connect
				self._drum.resetDevice(function(err) {
					if(err)
						console.log(err);

					setTimeout(function() {
						self.ConnectToDevice(callback);
					}, 200);
				});

				return (typeof callback === 'function' && callback(err));
			} else {

				self._connected = true;
				typeof callback === 'function' && callback(null);
			}
		});
	}



	self.EraseDevice = function(callback) {
		console.log('Erasing Flash...');

		self._drum.EraseFlash(function(err) {
			if(err) 
				return callback('Flash erase failed!');
		
			setTimeout(callback, 100);		
		});
	}



	self.ProgramDevice = function(samples, statusCallback, callback) {

		if(!self._connected)
			return (typeof callback === 'function' && callback('Erica Synths Pico Drum not connected!'));

		if(samples.length == 0)
			return (typeof callback === 'function' && callback('No samples on the list!'));


		typeof callback === 'function' && statusCallback('erase', 'Flash', 0, 0);

		// Erase memory
		self.EraseDevice(function(err) {
			if(err) 
				return (typeof callback === 'function' && callback(err));

			// Program all samples
			self._drum.ProgramVerifyAllSamples('program', samples, function(state, name, progress) {
				typeof callback === 'function' && statusCallback(state, name, progress, self._drum.ProgramStatus());
			}, function(err) {
				if(err)
					return (typeof callback === 'function' && callback(err));
				else
					typeof callback === 'function' && callback(err);
			});

		});
	}


	self.RestartDevice = function(callback) {
		// Reset 
		self._drum.resetDevice(function(err) {
			if(err)
				return (typeof callback === 'function' && callback(err));
	
		typeof callback === 'function' && callback(null);
		});
	}



	self.Init();
	
	return self;
}]);