app.factory('sampleService', [function() {
	
	var self = {};

	self._audioCtx = null;
	self._file = null;
	self._source = null;

	
	
	self.Init = function() {
		self._audioCtx = new (window.AudioContext || window.webkitAudioContext)();
	}


	self.Open = function(entry, callback) {
		
		if(!entry) {
			return (typeof callback === 'function' && callback('No entry!'));
		}

		var file = new File();

		// Read entry
		file.Read(entry, function(err, data) {
			if(err)
				return (typeof callback === 'function' && callback('Could not read file!'));


			typeof callback === 'function' && callback(null, data);

			// Decode
			// self._audioCtx.decodeAudioData(data, function(audioData) {
			// 	if(!audioData) {
			// 		return (callback === 'function' && callback('Error decoding audio data!'));
			// 	}

			// 	typeof callback === 'function' && callback(null, entry, audioData);

			// }).catch(function(err){
			// 	typeof callback === 'function' && callback(err);
			// });
		
		});
	}



	self.Decode = function(rawAudioData, callback) {

		self._audioCtx.decodeAudioData(rawAudioData, function(audioData) {
			if(!audioData) {
				return (callback === 'function' && callback('Error decoding audio data!'));
			}

			typeof callback === 'function' && callback(null, audioData);

		}).catch(function(err){
			typeof callback === 'function' && callback(err);
		});
	}



	self.Convert = function(data, callback) {

		if(!data) {
			return  (typeof callback === 'function' && callback('No data!'));
		}

		var rawData = data.getChannelData(0);

		// Resample
		self.Resample(rawData, data.getChannelData(0).length, function(audioData) {


			// Calculate new length (aligned to 4096 bytes length -> 2048 because int16)
			var sampleBufferLength = audioData.length;
			if((audioData.length % 2048) != 0) {
				sampleBufferLength += 2048 - (audioData.length % 2048);
			}
			
			// Convert form float32 to int16
			var sampleBuffer = new Int16Array(sampleBufferLength);
			for(var i=0; i<sampleBufferLength; i++) {
				sampleBuffer[i] = audioData[i] < 0 ? audioData[i] * 0x8000 : audioData[i] * 0x7FFF;
			}


			console.log('Sample length x2: ', audioData.length*2);
			console.log('Aligned length: ', sampleBuffer.byteLength)

			callback(null, sampleBuffer.buffer);
		});
	}


	self.TestWrite = function(name, data, callback) {

		var file = new File();

		file.Write(name, data, function(){
			if(err)
				return (typeof callback === 'function' && callback(err));

			return (typeof callback === 'function' && callback(null));
		});

	}



	self.Resample = function(data, length, callback) {


		// Check if resampling is needed
		if(self._audioCtx.sampleRate == 44100) {
			return (typeof callback === 'function' && callback(data));
		}

		// Do resampling
		 var o = new OfflineAudioContext(1, length*(44100/self._audioCtx.sampleRate), 44100);

		 // create audio buffer
  		var b = o.createBuffer(1, length, self._audioCtx.sampleRate);

      	b.getChannelData(0).set(data);


		/* Play it from the beginning. */
  		var source = o.createBufferSource();
		source.buffer = b;
		source.connect(o.destination);
		source.start(0);
		o.oncomplete = function(audioBuffer) {

		    // var aData = audioBuffer.renderedBuffer.getChannelData(0);
		    // var max = -Infinity;
		    // var min = +Infinity;
		    // for(var i=0; i<aData.length; i++) {
		    // 	if(aData[i] > max) max = aData[i];
		    // 	if(aData[i] < min) min = aData[i];
		    // }
		    // console.log(min, max);
		    // if(min < -1.0) console.log('Too LOW!!!!');
		    // if(max > 1.0) console.log('Too HIGH!!!!!');

			callback(audioBuffer.renderedBuffer.getChannelData(0));
  		}

  		/* Start rendering as fast as the machine can. */
  		o.startRendering();
	}



	self.Play = function(audioBuffer, callback) {
		self._source = self._audioCtx.createBufferSource();
		self._source.buffer = audioBuffer;
		self._source.connect(self._audioCtx.destination);
		// Set callback if defined
		if(typeof callback == 'function') {
			self._source.onended = callback;
		}
		self._source.start(0);
	}

	self.Stop = function() {
		self._source.stop();
	}




	self.GetSamplePackList = function(callback) {

		var xmlhttp = new XMLHttpRequest();
		var url = 'http://data.ericasynths.lv/picodrum/pack_list.json' + '?_=' + new Date().getTime();

		xmlhttp.onreadystatechange = function() {
			if(xmlhttp.readyState == 4) {
				if(xmlhttp.status == 200)
					typeof callback === 'function' && callback(null, JSON.parse(xmlhttp.responseText));
				else
					typeof callback === 'function' && callback('Failed to download sample list! Error: ', xmlhttp.status);
			}
		};
	
		xmlhttp.open("GET", url);
		xmlhttp.send();

	}



	self.DownloadSamplePack = function(file, callback) {

		var xmlhttp = new XMLHttpRequest();
		var url = 'http://data.ericasynths.lv/picodrum/' + file;

		xmlhttp.onreadystatechange = function() {
			if(xmlhttp.readyState == 4) {
				if(xmlhttp.status == 200)
					typeof callback === 'function' && callback(null, xmlhttp.response);
				else
					typeof callback === 'function' && callback('Failed to download sample pack! Error: ', xmlhttp.status);
			}
		};
	
		xmlhttp.responseType = 'arraybuffer';
		xmlhttp.open("GET", url);
		xmlhttp.send();
	}



	self.Init();
	
	return self;
}]);
